unit Capi2;
{**************************************************************************}
{ DELPHI interface to the CAPI 2 DLL for Windows 9x/NT Applications		}
{ Modified from a unit Copyright (c) 1996 Alexander Staubo	(alex@mop.no) by}
{ Austin Kinsella (Kinsella@ITCarlow.ie) Institute of Technology Carlow	}
{ May 1999																	}
{**************************************************************************}
{From the COMMON-ISDN-API Version 2.0 Aug 1997 docs - Part 2, OS Specific	}
{NT & Win9x are the same for application calls:							}
{--------------------------------------------------------------------------}
{The DLL providing the function interface has to be named "CAPI2032.DLL". 	}
{It is a 32-bit DLL exporting 32-bit APIENTRY type functions. 				}
{The DLL functions are exported under following names and ordinal numbers: }
{	CAPI_MANUFACTURER (reserved) CAPI2032.99								}
{	CAPI_REGISTER CAPI2032.1												}
{	CAPI_RELEASE CAPI2032.2                                                 }
{	CAPI_PUT_MESSAGE CAPI2032.3                                             }
{	CAPI_GET_MESSAGE CAPI2032.4                                             }
{	CAPI_WAIT_FOR_SIGNAL CAPI2032.5                                         }
{	CAPI_GET_MANUFACTURER CAPI2032.6                                        }
{	CAPI_GET_VERSION CAPI2032.7                                             }
{	CAPI_GET_SERIAL_NUMBER CAPI2032.8                                       }
{	CAPI_GET_PROFILE CAPI2032.9                                             }
{	CAPI_INSTALLED CAPI2032.10                                              }
{These functions can be called by an application according to the DLL		}
{conventions as imported functions. In the Windows NT environment following}
{types are used to define the functional interface:                        }
{	WORD 16-bit unsigned integer	                                        }
{	DWORD 32-bit unsigned integer									        }
{	PVOID pointer to any memory location                                    }
{	PVOID * pointer to a PVOID                                              }
{	char * pointer to a character string                                    }
{	DWORD * pointer to a 32-bit unsigned integer value                      }
{--------------------------------------------------------------------------}
{For WORD use Pascal Word. A DWORD type is declared as LongInt so be need	}
{be careful if used in calculations.										}
{In general pointer parameters are dealt with by declaring them as vars	}
{in the function headers.                                                  }
{**************************************************************************}
{}
interface

type
  DWord 	= Longint;
  SubCmd	= (REQ, CONF, IND, RESP); {When high bit is set these become values}
  									  {for subcommand field. Use CodeSubCommand}
  ParameterType	= (parWord, parDWord, parStruct); {see BuildMessage below}
const
{Command field in header. xxxx shows subcommands taken: Req, Conf, Ind, Resp}
	cmd_ALERT					= $01; {1100}
	cmd_CONNECT					= $02; {1111}
	cmd_CONNECT_ACTIVE			= $03; {0011}
	cmd_DISCONNECT				= $04; {1111}
	cmd_LISTEN					= $05; {1100}
	cmd_INFO					= $08; {1111}
	cmd_SELECT_B_PROTOCOL		= $41; {0011}
	cmd_FACILITY				= $80; {1111}
	cmd_CONNECT_B3				= $82; {1111}
	cmd_CONNECT_B3_ACTIVE		= $83; {0011}
	cmd_DISCONNECT_B3			= $84; {1111}
	cmd_DATA_B3					= $86; {1111}
	cmd_RESET_B3				= $87; {1111}
	cmd_CONNECT_B3_T90_ACTIVE	= $88; {0011}
	cmd_MANUFACTURER			= $FF; {1111}
{Names for subcommands if we need to display or log them}
	SubCmdName : Array [SubCmd] of String
  				= ('REQ', 'CONF', 'IND', 'RESP');
type
	CapiMessagePtr = ^CapiMessage; {Actual ISDN interraction is through GET and	}
   							   {PUT message. Messages have a common header	}
                                  {structure. Structure of Body depends on the	}
                                  {specific Command/Subcommand					}
	CapiMsgHdr = record
  		Length			: Word;
   	APPL_ID			: Word;
   	Command			: Byte;
   	SubCommand		: Byte;
   	MessageNumber	: Word;
   	end; {CapiMsgHdr}
   CapiMessage	= record
   	Header	: CapiMsgHdr;
       Body	: Array  [0..1000] of Byte; {We don't need this big an array	}
       									{but we don't know the max size		}
                                           {In practice, create a buffer of 	}
                                           {suitable size, a variable of type	}
                                           {CapiMessagePtr, then load CMP with }
                                           {the buffer address CMP := @Buffer	}
       end; {CapiMessage}
   ByteArray64	= Array [0..63] of Byte;{various buffers for info}
   CharArray64	= Array [0..63] of Char;
   CharArray8	= Array [0..7] of Char;
Var
	ApplicationId	: DWord; {declared here 'cos it is persistent while we run	}
   						 {CAPI sets it when we register and it is used in 	}
                            {calls passed to ISDN								}
{******************************************************************************}
{ 					Common ISDN API (CAPI) functions 							}
{ Use StdCall to force C arg passing (rightmost first) and DLL cleans stack	}
{ The functions return a 32-bit error code, but only the low 16 bits are		}
{ used																			}
{******************************************************************************}

function CAPI_REGISTER (MessageBufferSize 		: DWord;
                       MaxLogicalConnections	: DWord;
                       MaxBDataBlocks			: DWord;
                       MaxBDataLen				: DWord;
                       var ApplicationID 		: DWord) : DWord; StdCall;
{Page 90: typically memory required should be calculated as:					}
{ MessageBufferSize = 1024 + (1024 * maxLogicalConnection)						}
{ maxLogicalConnection - max no of concurrent connections						}
{ maxBDataBlocks - max no of received data blocks that can be reported 		}
{ simultaneously for each connection. should be between 2 and 7.				}
{ maxBDataLen - max size of the application data block to be transmitted		}
{ and received. Default for ISO 7776 (X.75) is 128 octets. up to 2048			}
{ ApplicationID - returned value for use in other calls						}
function CAPI_RELEASE (ApplicationID : DWord) : DWord; StdCall;
function CAPI_PUT_MESSAGE (ApplicationID : DWord;
							CapiMsg : CapiMessagePtr) : DWord; StdCall;
function CAPI_GET_MESSAGE (ApplicationID : DWord;
							var CapiMsg : CapiMessagePtr) : DWord; StdCall;
function CAPI_WAIT_FOR_SIGNAL (ApplicationID : DWord) : DWord; StdCall;
Procedure CAPI_GET_MANUFACTURER (var Identification : CharArray64); StdCall;
function CAPI_GET_VERSION (var CapiMajorVer : DWord;
							var CapiMinorVer : DWord;
                           var ManfMajorVer : DWord;
                           var ManfMinorVer : DWord) : DWord; StdCall;
function CAPI_GET_SERIAL_NUMBER (var Serial : CharArray8) : DWord; StdCall;
function CAPI_INSTALLED : DWord; StdCall;
function CAPI_GET_PROFILE (var Profile : ByteArray64;
							Controller : DWord): DWord; StdCall;
function CAPI_MANUFACTURER (ApplicationID : Word) : DWord; StdCall;

{ CAPI utility functions }

Function DecodeCommand (Command : Byte) : String;
{Turn a command code into a name for logging or display purposes}
Function CodeSubCommand (Which : SubCmd) : Byte;
{encode REQ, CONF, IND or RESP as a byte with MSB set ie x80, x81, x82 or x83}
Function DecodeSubCommand (SubCommand : Byte) : String;
{return the name of the subcommand}
Function BuildMessage (TheMessage : CapiMessagePtr;
						OffSet : Integer;
                       const AddOn;
                       TypeOfAddOn : ParameterType) : Integer;
{We need to put a succession of parameters of different types into the message	}
{in the sequence required by the command. This routine allows us to move them 	}
{in to the next position in the message body and updates the Length field in 	}
{the Header. The position is maintained by OffSet, which should be set to 0 	}
{before the first call in the sequence. The caller should fill in the other 	}
{header details. No checking is done so caller should ensure that enough room	}
{remains in TheMessage^ to take the AddOn. TypeOfAddOn is parWord, parDWord	}
{or parStruct. If parStruct, AddOn should be a string, and						}
{BuildMessage will include the size byte needed for the struct. This works for	}
{empty structs too. AddOn will be inserted into the message body at OffSet		}
Function DecodeMessage (TheMessage : CapiMessagePtr;
						OffSet : Integer;
                       var TheParameter;
                       TypeOfAddOn : ParameterType) : Integer;
{See comment on BuildMessage. Call Decode message enough times to get to the 	}
{parameter you want. Offset should be 0 for the first call. Caller could set	}
{Offset non-zero to get a specific parameter, but if there are structs you can	}
{not predict what length they will be, so it is better to make a call for each	}
{parameter																		}

implementation
uses
  SysUtils;
const
  DllName = 'capi2032.dll';

{ Externals }
function CAPI_REGISTER; external DllName;
function CAPI_RELEASE; external DllName;
function CAPI_GET_MESSAGE; external DllName;
function CAPI_PUT_MESSAGE; external DllName;
function CAPI_WAIT_FOR_SIGNAL; external DllName;
Procedure CAPI_GET_MANUFACTURER; external DllName;
function CAPI_GET_VERSION; external DllName;
function CAPI_GET_SERIAL_NUMBER; external DllName;
function CAPI_GET_PROFILE; external DllName;
function CAPI_INSTALLED; external DllName;
function CAPI_MANUFACTURER; external DllName;

{ Publics }
Function DecodeCommand (Command : Byte) : String;
{Supply a name to match code}
Begin {CommandDecode}
Case Command of
	$01 : Result := 'ALERT';
	$02 : Result := 'CONNECT';
	$03 : Result := 'CONNECT_ACTIVE';
	$04 : Result := 'DISCONNECT';
	$05 : Result := 'LISTEN';
	$08 : Result := 'INFO';
	$41 : Result := 'SELECT_B_PROTOCOL';
	$80 : Result := 'FACILITY';
	$82 : Result := 'CONNECT_B3';
	$83 : Result := 'CONNECT_B3_ACTIVE';
	$84 : Result := 'DISCONNECT_B3';
	$86 : Result := 'DATA_B3';
	$87 : Result := 'RESET_B3';
	$88 : Result := 'CONNECT_B3_T90_ACTIVE';
	$FF : Result := 'MANUFACTURER';
   End; {Case Command}
End; {DecodeCommand}

Function CodeSubCommand (Which : SubCmd) : Byte;
{encode REQ, CONF, IND or RESP as a byte with MSB set ie x80, x81, x82 or x83}
Begin {CodeSubCommand}
Result := Byte (128 + Ord (Which));
End; {CodeSubCommand}

Function DecodeSubCommand (SubCommand : Byte) : String;
{return the name of the subcommand}
Begin {DecodeSubCommand}
{strip off high nibble, typecast to SubCmd, and use as index into name array}
Result := SubCmdName [ SubCmd(SubCommand AND $0F)];
End; {DecodeSubCommand}

Function BuildMessage (TheMessage : CapiMessagePtr;
						OffSet : Integer;
                       const AddOn;
                       TypeOfAddOn : ParameterType) : Integer;
Type
	ArrayCast = Array [0..3] of Byte;
Var
	Step	: Integer;
Begin {BuildMessage}
With TheMessage^ do Case TypeOfAddOn of
	parWord		:	Begin {AddOn is Word}
   				For Step := 0 to 1 do
   					Body [Offset + Step] := ArrayCast(AddOn)[Step];
   				OffSet := OffSet + 2;
					End; {AddOn is Word}
   parDWord	:	Begin {AddOn is Double}
   				For Step := 0 to 3 do
   					Body [Offset + Step] := ArrayCast(AddOn)[Step];
   				OffSet := OffSet + 4;
   				End; {AddOn is Double}
   parStruct	:	Begin {AddOn is Struct}
                   Body [Offset] := Length (String(AddOn));
   				For Step := 1 to Length (String(AddOn)) do
   					Body [Offset + Step] := Ord(String(AddOn)[Step]);
   				OffSet := OffSet + Length (String (AddOn)) + 1;
   				End; {AddOn is Struct}
   End; {Case TypeOfAddOn}
TheMessage^.Header.Length := SizeOf (CapiMsgHdr) + Offset;
Result := Offset;
End; {BuildMessage}

Function DecodeMessage (TheMessage : CapiMessagePtr;
						OffSet : Integer;
                       var TheParameter;
                       TypeOfAddOn : ParameterType) : Integer;
Type
	ArrayCast = Array [0..3] of Byte;
Var
	Step	: Integer;
Begin {DecodeMessage}
With TheMessage^ do Case TypeOfAddOn of
	parWord		:	Begin {TheParameter is Word}
   				For Step := 0 to 1 do
   					ArrayCast(TheParameter)[Step] := Body [Offset + Step];
   				OffSet := OffSet + 2;
					End; {TheParameter is Word}
   parDWord	:	Begin {TheParameter is Double}
   				For Step := 0 to 3 do
   					ArrayCast(TheParameter)[Step] := Body [Offset + Step];
   				OffSet := OffSet + 4;
   				End; {TheParameter is Double}
   parStruct	:	Begin {TheParameter is Struct}
                   SetLength (String(TheParameter), Body [Offset]);
   				For Step := 1 to Length (String(TheParameter)) do
   					String(TheParameter)[Step] := Chr (Body [Offset + Step]);
   				OffSet := OffSet + Length (String(TheParameter)) + 1;
   				End; {TheParameter is Struct}
   End; {Case TypeOfAddOn}
Result := Offset;
End; {DecodeMessage}

end. {Capi2}
